﻿//
// Copyright (c) 2008-2011, Kenneth Bell
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

namespace DiscUtils.Vhdx
{
    using System;
    using System.Collections.Generic;
    using System.Text;

    internal sealed class RegionTable : IByteArraySerializable
    {
        public const uint RegionTableSignature = 0x69676572;
        public const int FixedSize = (int)(64 * Sizes.OneKiB);
        public static readonly Guid BatGuid = new Guid("2DC27766-F623-4200-9D64-115E9BFD4A08");
        public static readonly Guid MetadataRegionGuid = new Guid("8B7CA206-4790-4B9A-B8FE-575F050F886E");

        private byte[] _data;

        public uint Signature;
        public uint Checksum;
        public uint EntryCount;
        public uint Reserved;
        public IDictionary<Guid, RegionEntry> Regions;

        public int Size
        {
            get { return FixedSize; }
        }

        public bool IsValid
        {
            get
            {
                if (Signature != RegionTableSignature)
                {
                    return false;
                }

                if (EntryCount > 2047)
                {
                    return false;
                }

                byte[] checkData = new byte[FixedSize];
                Array.Copy(_data, checkData, FixedSize);
                Utilities.WriteBytesLittleEndian((uint)0, checkData, 4);
                return Checksum == Crc32LittleEndian.Compute(Crc32Algorithm.Castagnoli, checkData, 0, FixedSize);
            }
        }

        public int ReadFrom(byte[] buffer, int offset)
        {
            _data = new byte[FixedSize];
            Array.Copy(buffer, offset, _data, 0, FixedSize);

            Signature = Utilities.ToUInt32LittleEndian(_data, 0);
            Checksum = Utilities.ToUInt32LittleEndian(_data, 4);
            EntryCount = Utilities.ToUInt32LittleEndian(_data, 8);
            Reserved = Utilities.ToUInt32LittleEndian(_data, 12);
            Regions = new Dictionary<Guid, RegionEntry>();

            if (IsValid)
            {
                for (int i = 0; i < EntryCount; ++i)
                {
                    RegionEntry entry = Utilities.ToStruct<RegionEntry>(_data, 16 + (32 * i));
                    Regions.Add(entry.Guid, entry);
                }
            }

            return Size;
        }

        public void WriteTo(byte[] buffer, int offset)
        {
            throw new NotImplementedException();
        }
    }
}
